/*
 * Copyright (C) 2022 Ortega Froysa, Nicolás <nicolas@ortegas.org> All rights reserved.
 * Author: Ortega Froysa, Nicolás <nicolas@ortegas.org>
 *
 * This software is provided 'as-is', without any express or implied
 * warranty. In no event will the authors be held liable for any damages
 * arising from the use of this software.
 *
 * Permission is granted to anyone to use this software for any purpose,
 * including commercial applications, and to alter it and redistribute it
 * freely, subject to the following restrictions:
 *
 * 1. The origin of this software must not be misrepresented; you must not
 *    claim that you wrote the original software. If you use this software
 *    in a product, an acknowledgment in the product documentation would be
 *    appreciated but is not required.
 *
 * 2. Altered source versions must be plainly marked as such, and must not be
 *    misrepresented as being the original software.
 *
 * 3. This notice may not be removed or altered from any source
 *    distribution.
 */

extern crate rand;

fn main() {
	let size = 100;

	// initialize the boxes with randomly assigned numbers
	let mut boxes:Vec<u32> = Vec::with_capacity(size as usize);

	{
		let mut nums:Vec<u32> = (0..size).collect();

		while nums.len() > 0
		{
			let rand_i:usize = rand::random::<usize>() % nums.len();
			boxes.push(nums[rand_i]);
			nums.remove(rand_i);
		}
	}

	// have prisoners find their number
	let mut all_found = true;
	for i in 0..size
	{
		let mut next_box:usize = i as usize;
		let mut found = false;
		for _j in 0..(size/2)
		{
			if boxes[next_box] == i
			{
				found = true;
				break;
			}
			else
			{
				next_box = boxes[next_box] as usize;
			}
		}
		if !found
		{
			all_found = false;
		}
	}

	// print result
	if all_found
	{
		println!("All prisoners found their numbers!");
	}
	else
	{
		println!("Some prisoners didn't find their numbers!");
	}
}
